package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.RequestHeader;
import cn.rankin.apiweb.entity.GlobalHeader;
import cn.rankin.apiweb.helper.MacAddressHelper;
import cn.rankin.apiweb.service.user.UserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.util.HttpUtil;
import cn.rankin.data.api.app.dto.DeviceLoginDTO;
import cn.rankin.data.api.app.dto.LoginInfoDTO;
import cn.rankin.data.api.app.vo.UserInfoVo;
import eu.bitwalker.useragentutils.Browser;
import eu.bitwalker.useragentutils.OperatingSystem;
import eu.bitwalker.useragentutils.UserAgent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
@RestController
@RequestMapping(value = "/token")
public class LoginController
{

    @Autowired
    private UserService userService;

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<UserInfoVo> login(HttpServletRequest request, @Valid @RequestBody LoginInfoDTO loginInfoDTO)
    {

        String terminal = request.getHeader("terminal");
        String merchant = request.getHeader("merchant");
        String userAgent = request.getHeader("User-Agent");


        String ip = HttpUtil.getClientIp(request);
        loginInfoDTO.setIp(ip);
        loginInfoDTO.setTerminal(terminal);
        loginInfoDTO.setMerchant(merchant);

        log.info("login request header : ip={},terminal={},merchant={},user-agent={}", ip, terminal, merchant,userAgent);
        //填充设备信息
        fillDeviceInfo(userAgent, loginInfoDTO);

        return userService.login(loginInfoDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<UserInfoVo> refresh(HttpServletRequest request, @RequestBody DeviceLoginDTO deviceLoginDTO)
    {
        String deviceCode = deviceLoginDTO.getDeviceCode();
        String merchant = request.getHeader("merchant");
        String userAgent = request.getHeader("User-Agent");
        log.info("refresh request header : deviceCode={}, merchant={},userAgent={}", deviceCode, merchant,userAgent);

        //填充设备信息
        fillDeviceInfo(userAgent, deviceLoginDTO);

        return userService.refresh(deviceCode, merchant, Boolean.TRUE,deviceLoginDTO);
    }

    @RequestMapping(method = RequestMethod.DELETE)
    public APIResult<Boolean> logout(@RequestHeader GlobalHeader globalHeader)
    {
        String userId = globalHeader.getUid();
        return userService.logout(userId);
    }

    /**
     * Fill device info
     * @param userAgentStr the string of user agent
     * @param loginInfoDTO the login info
     */
    private void fillDeviceInfo(String userAgentStr, DeviceLoginDTO loginInfoDTO)
    {
        if (userAgentStr == "" || userAgentStr == null)
        {
            return;
        }
        if (loginInfoDTO == null)
        {
            return;
        }

        UserAgent userAgent = UserAgent.parseUserAgentString(userAgentStr);

        //获取浏览器对象
        Browser browser = userAgent.getBrowser();

        //获取操作系统对象
        OperatingSystem operatingSystem = userAgent.getOperatingSystem();

        //操作系统名称
        String osName = operatingSystem.getName();
        loginInfoDTO.setDeviceName(osName);

        //获取厂商
        String osMfrs = operatingSystem.getManufacturer().toString();
        loginInfoDTO.setDeviceMfrs(osMfrs);

        //获取设备类型
        String deviceType = operatingSystem.getDeviceType().toString();
        loginInfoDTO.setDeviceType(deviceType);

        //获取设备型号
        //获取设备型号
        if ("MOBILE".equals(deviceType))
        {
            String split[] = userAgentStr.split("Build/");

            if(split!=null || split.length > 0)
            {
                String splitType[] =  split[0].split(";");
                if(splitType!=null && splitType.length>0)
                {
                    loginInfoDTO.setDeviceModel(splitType[splitType.length-1]);
                }

            }

        }
        else if ("TABLET".equals(deviceType))
        {
            String split[] = userAgentStr.split("Build/");

            if(split!=null || split.length > 0)
            {
                String splitType[] =  split[0].split(";");
                if(splitType!=null && splitType.length>0)
                {
                    loginInfoDTO.setDeviceModel(splitType[splitType.length-1]);
                }

            }
        }
        else
        {
            loginInfoDTO.setDeviceModel(osName);
        }
    }

    /**
     * Fill device info
     * @param userAgentStr the string of user agent
     * @param loginInfoDTO the login info
     */
    private void fillDeviceInfo(String userAgentStr, LoginInfoDTO loginInfoDTO)
    {
        if (userAgentStr == "" || userAgentStr == null)
        {
            return;
        }
        if (loginInfoDTO == null)
        {
            return;
        }

        UserAgent userAgent = UserAgent.parseUserAgentString(userAgentStr);

        //获取浏览器对象
        Browser browser = userAgent.getBrowser();

        //获取操作系统对象
        OperatingSystem operatingSystem = userAgent.getOperatingSystem();

        //操作系统名称
        String osName = operatingSystem.getName();
        loginInfoDTO.setDeviceName(osName);

        //获取厂商
        String osMfrs = operatingSystem.getManufacturer().toString();
        loginInfoDTO.setDeviceMfrs(osMfrs);

        //获取设备类型
        String deviceType = operatingSystem.getDeviceType().toString();
        loginInfoDTO.setDeviceType(deviceType);

        //获取设备型号
        if ("MOBILE".equals(deviceType))
        {
            String split[] = userAgentStr.split("Build/");

            if(split!=null || split.length > 0)
            {
                String splitType[] =  split[0].split(";");
                if(splitType!=null && splitType.length>0)
                {
                    loginInfoDTO.setDeviceModel(splitType[splitType.length-1]);
                }

            }

        }
        else if ("TABLET".equals(deviceType))
        {
            String split[] = userAgentStr.split("Build/");

            if(split!=null || split.length > 0)
            {
                String splitType[] =  split[0].split(";");
                if(splitType!=null && splitType.length>0)
                {
                    loginInfoDTO.setDeviceModel(splitType[splitType.length-1]);
                }

            }
        }
        else
        {
            loginInfoDTO.setDeviceModel(osName);
        }
    }

}
