package cn.rankin.cmsweb.controller.stmt;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.stmt.StmtCampusService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.common.utils.util.ProvinceUtil;
import cn.rankin.data.api.user.dto.CampusSearchDTO;
import cn.rankin.data.api.user.vo.CampusVo;
import org.apache.poi.hssf.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping(value = "/stmt/campus")
public class StmtCampusController
{
    @Autowired
    private StmtCampusService stmtService;

    /**
     * 查询已开通校区报表
     *
     * @param user            the user info
     * @param campusSearchDTO the campus search
     * @return
     */
    @RequestMapping(value = {"/list", "/"}, method = RequestMethod.GET)
    public APIResult<Page<CampusVo>> campusList(@NeedUser UserDetails user, CampusSearchDTO campusSearchDTO)
    {
        if (!user.isPlatForm())
        {
            String merchantId = user.getMerchantId();
            campusSearchDTO.setMerchantId(merchantId);
        }
        APIResult<Page<CampusVo>> pageData = stmtService.search(BeanUtil.convertToMap(campusSearchDTO));

        //设置省份
        if (pageData != null)
        {
            List<CampusVo> list = pageData.getData().getList();
            if (list != null && list.size() > 0)
            {
                for (int i = 0; i < list.size(); i++)
                {
                    CampusVo vo = list.get(i);
                    String provinceCode = vo.getProvinceCode();
                    String provinceName = ProvinceUtil.get(provinceCode);
                    vo.setProvinceName(provinceName);
                }
            }
        }

        return pageData;
    }

    /**
     * 已开通校区报表导出
     *
     * @param user            the user info
     * @param campusSearchDTO the campus search
     * @return
     */
    @RequestMapping(value = {"/export", "/"}, method = RequestMethod.GET)
    public String exportCampus(HttpServletResponse res, @NeedUser UserDetails user, CampusSearchDTO campusSearchDTO)
    {
        if (!user.isPlatForm())
        {
            String merchantId = user.getMerchantId();
            campusSearchDTO.setMerchantId(merchantId);
        }

        //不是条件查询就全部下载
        String code = campusSearchDTO.getCode();
        String name = campusSearchDTO.getName();
        if ((code != null && code != "") || (name != null && name != ""))
        {

        }
        else
        {
            campusSearchDTO.setPageNo(1);
            campusSearchDTO.setPageSize(100000);
        }

        APIResult<List<CampusVo>> apiResult = stmtService.query(BeanUtil.convertToMap(campusSearchDTO));

        List<CampusVo> list = apiResult.getData();
        //创建HSSFWorkbook对象(excel的文档对象)
        HSSFWorkbook wb = new HSSFWorkbook();
        //建立新的sheet对象（excel的表单）
        HSSFSheet sheet = wb.createSheet("已开通校区报表");
        //设置单元格宽度
        sheet.setColumnWidth(0, 15 * 256);
        sheet.setColumnWidth(1, 50 * 256);
        sheet.setColumnWidth(2, 10 * 256);
        sheet.setColumnWidth(3, 10 * 256);
        sheet.setColumnWidth(4, 30 * 256);
        sheet.setColumnWidth(5, 10 * 256);
        sheet.setColumnWidth(6, 15 * 256);
        sheet.setColumnWidth(7, 15 * 256);
        //在sheet里创建第一行，参数为行索引(excel的行)，可以是0～65535之间的任何一个
        HSSFRow row1 = sheet.createRow(0);
        //创建单元格并设置单元格内容
        row1.createCell(0).setCellValue("校区编号");
        row1.createCell(1).setCellValue("校区名称");
        row1.createCell(2).setCellValue("客户类型");
        row1.createCell(3).setCellValue("所属省");
        row1.createCell(4).setCellValue("所属市");
        row1.createCell(5).setCellValue("联系人");
        row1.createCell(6).setCellValue("联系方式");
        row1.createCell(7).setCellValue("更新时间");

        //单元格日期格式
        HSSFCellStyle cellStyle = wb.createCellStyle();
        HSSFDataFormat format = wb.createDataFormat();
        cellStyle.setDataFormat(format.getFormat("yyyy/mm/dd"));

        //插入数据
        if (list != null && list.size() > 0)
        {
            for (int i = 0; i < list.size(); i++)
            {

                CampusVo vo = list.get(i);
                String provinceCode = vo.getProvinceCode();
                String provinceName = ProvinceUtil.get(provinceCode);
                vo.setProvinceName(provinceName);

                HSSFRow row = sheet.createRow(i + 1);
                row.createCell(0).setCellValue(vo.getCode());
                row.createCell(1).setCellValue(vo.getName());
                row.createCell(2).setCellValue(vo.getMerchantName());
                row.createCell(3).setCellValue(vo.getProvinceName());
                row.createCell(4).setCellValue(vo.getCityName());
                row.createCell(5).setCellValue(vo.getContactName());
                row.createCell(6).setCellValue(vo.getMobile());


                HSSFCell cell = row.createCell(7);
                cell.setCellValue(vo.getGmtModified());
                cell.setCellStyle(cellStyle);
            }
        }

        try
        {
            long currentTime = new Date().getTime();
            //输出Excel文件
            OutputStream output = res.getOutputStream();
            res.reset();
            res.setHeader("Content-disposition", "attachment; filename=lingjiao_campus_" + currentTime + ".xls");
            res.setContentType("application/msexcel");
            wb.write(output);
            output.close();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        return null;
    }

}
