package cn.rankin.cmsweb.controller.stmt;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.product.ProductService;
import cn.rankin.cmsweb.service.stmt.StmtCampusService;
import cn.rankin.cmsweb.service.user.CampusService;
import cn.rankin.cmsweb.service.user.TerminalUserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.common.utils.util.DateUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.vo.ProductVo;
import cn.rankin.data.api.user.dto.CampusSearchDTO;
import cn.rankin.data.api.user.dto.TerminalUserSearchDTO;
import cn.rankin.data.api.user.vo.CampusVo;
import cn.rankin.data.api.user.vo.TerminalUserAuthVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping(value = "/stmt/terminal/user")
public class StmtTerminalUserController
{
    @Autowired
    private StmtCampusService StmtService;

    @Autowired
    private TerminalUserService terminalUserService;

    @Autowired
    private ProductService productService;

    @Autowired
    private CampusService campusService;

    /**
     * 查询已开通校区报表
     *
     * @param user      the user info
     * @param searchDTO the campus search
     * @return
     */
    @RequestMapping(value = {"/page", "/"}, method = RequestMethod.GET)
    public APIResult<Page<TerminalUserAuthVo>> campusList(@NeedUser UserDetails user, TerminalUserSearchDTO searchDTO)
    {
        if (!user.isPlatForm())
        {
            String merchantId = user.getMerchantId();
            searchDTO.setMerchantId(merchantId);
        }
        Page<TerminalUserAuthVo> authVoPage = terminalUserService.findAuthList(BeanUtil.convertToMap(searchDTO));

        List<TerminalUserAuthVo> terminalUserAuthVos = authVoPage.getList();
        setProductInfo(terminalUserAuthVos);
        setCampusInfo(terminalUserAuthVos);

        return APIResult.ok(authVoPage);

    }

    /**
     * 查询已开通校区报表
     *
     * @param user      the user info
     * @param searchDTO the campus search
     * @return
     */
    @RequestMapping(value = {"/export", "/"}, method = RequestMethod.GET)
    public String exportCampus(HttpServletResponse res, @NeedUser UserDetails user, TerminalUserSearchDTO searchDTO)
    {
        log.info("fastExpired："+searchDTO.getFastExpired()+"");
        if (!user.isPlatForm())
        {
            String merchantId = user.getMerchantId();
            searchDTO.setMerchantId(merchantId);
        }

        //不是条件查询就全部下载
        String code = searchDTO.getCode();
        if (code == null || code == "")
        {
            searchDTO.setPageNo(1);
            searchDTO.setPageSize(100000);
        }

        Page<TerminalUserAuthVo> authVoPage = terminalUserService.findAuthList(BeanUtil.convertToMap(searchDTO));

        List<TerminalUserAuthVo> list = authVoPage.getList();
        setProductInfo(list);
        setCampusInfo(list);

        //创建HSSFWorkbook对象(excel的文档对象)
        HSSFWorkbook wb = new HSSFWorkbook();
        //建立新的sheet对象（excel的表单）
        HSSFSheet sheet = wb.createSheet("已开通账户报表");
        //设置单元格宽度
        sheet.setColumnWidth(0, 20 * 256);
        sheet.setColumnWidth(1, 10 * 256);
        sheet.setColumnWidth(2, 30 * 256);
        sheet.setColumnWidth(3, 30 * 256);
        sheet.setColumnWidth(4, 10 * 256);
        sheet.setColumnWidth(5, 30 * 256);
        sheet.setColumnWidth(6, 15 * 256);
        sheet.setColumnWidth(7, 15 * 256);
        //在sheet里创建第一行，参数为行索引(excel的行)，可以是0～65535之间的任何一个
        HSSFRow row1 = sheet.createRow(0);
        //创建单元格并设置单元格内容
        row1.createCell(0).setCellValue("终端编号");
        row1.createCell(1).setCellValue("课程包编号");
        row1.createCell(2).setCellValue("课程包名称");
        row1.createCell(3).setCellValue("权限有效期");
        row1.createCell(4).setCellValue("权限有效时长");
        row1.createCell(5).setCellValue("校区名称");
        row1.createCell(6).setCellValue("联系人");
        row1.createCell(7).setCellValue("联系方式");

        //单元格日期格式
        HSSFCellStyle cellStyle = wb.createCellStyle();
        HSSFDataFormat format = wb.createDataFormat();
        cellStyle.setDataFormat(format.getFormat("yyyy/mm/dd"));

        //插入数据
        if (list != null && list.size() > 0)
        {
            for (int i = 0; i < list.size(); i++)
            {

                TerminalUserAuthVo vo = list.get(i);

                HSSFRow row = sheet.createRow(i + 1);
                row.createCell(0).setCellValue(vo.getUCode());
                row.createCell(1).setCellValue(vo.getPCode());
                row.createCell(2).setCellValue(vo.getPName());
                String dateStr = "起始时间: " + DateUtil.parseDateStr(vo.getStartTime()) + "\n";
                dateStr += "到期时间：" + DateUtil.parseDateStr(vo.getEndTime());
                row.createCell(3).setCellValue(dateStr);

                //判断是否到期
                Long curTime = new Date().getTime();
                //获取到期天数
                String expiredStr = "已到期";
                long days = (vo.getEndTime() - curTime) / (1000 * 3600 * 24);
                if (days > 0)
                {
                    expiredStr = days + "天到期";
                }

                row.createCell(4).setCellValue(new HSSFRichTextString(expiredStr));
                row.createCell(5).setCellValue(vo.getCampusName());
                row.createCell(6).setCellValue(vo.getCampusContactName());
                row.createCell(7).setCellValue(vo.getCampusContactWay());
            }
        }

        try
        {
            long currentTime = new Date().getTime();
            //输出Excel文件
            OutputStream output = res.getOutputStream();
            res.reset();
            res.setHeader("Content-disposition", "attachment; filename=lingjiao_terminal_user_" + currentTime + ".xls");
            res.setContentType("application/msexcel");
            wb.write(output);
            output.close();
        }
        catch (Exception ex)
        {

        }

        return null;
    }

    private void setProductInfo(List<TerminalUserAuthVo> terminalUserAuthVos)
    {
        if (CollectionUtils.isEmpty(terminalUserAuthVos))
        {
            return;
        }

        List<ProductVo> productVos = productService.productList();
        if (CollectionUtils.isEmpty(productVos))
        {
            return;
        }

        Map<String, ProductVo> productVoMap = ListUtil.convert(productVos, "pid", ProductVo.class);

        terminalUserAuthVos.forEach(vo ->
        {
            ProductVo productVo = productVoMap.get(vo.getPid());
            if (null != productVo)
            {
                vo.setPCode(productVo.getCode());
                vo.setPName(productVo.getName());
            }
            else
            {
                log.info("not found product by pid ,pid={}", vo.getPid());
            }
        });

    }

    private void setCampusInfo(List<TerminalUserAuthVo> terminalUserAuthVos)
    {
        if (CollectionUtils.isEmpty(terminalUserAuthVos))
        {
            return;
        }

        for (TerminalUserAuthVo vo : terminalUserAuthVos)
        {
            //时间转换
            String startTime = DateUtil.parseDateStr(vo.getStartTime());
            String endTime = DateUtil.parseDateStr(vo.getEndTime());
            vo.setStartTimeStr(startTime);
            vo.setEndTimeStr(endTime);
            String campusId = vo.getCampusId();

            if (campusId != null && campusId.trim() != "")
            {
                APIResult<CampusVo> apiResult = campusService.getCampus(campusId);
                CampusVo campusVo = apiResult.getData();
                if (campusVo != null)
                {
                    vo.setCampusName(campusVo.getName());
                    vo.setCampusContactName(campusVo.getContactName());
                    vo.setCampusContactWay(campusVo.getMobile());
                }
            }
        }
    }

}
