package cn.rankin.userservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.EventTypeEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.user.entity.DownloadInfo;
import cn.rankin.data.api.user.entity.Resource;
import cn.rankin.data.api.user.entity.UserDevice;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

public interface UserDeviceRepository extends BasicJpaRepository<UserDevice, String> {

    @Query(value = "select device from UserDevice device where device.userId = ?1 AND device.deviceName= ?2 AND device.deviceModel = ?3")
    UserDevice findUserDevice(String uid, String deviceName,String deviceModel);

    @Transactional
    @Modifying
    @Query(value = "update UserDevice device set device.num = ?4 where device.userId = ?1 AND device.deviceName= ?2 AND device.deviceModel = ?3")
    int updateDevice(String uid, String deviceName,String deviceModel,int num);

    /**
     * 根据用户编号查询设备信息
     * @param userId the id of user
     * @return
     */
    @Query(value = "select u from  UserDevice  u where u.userId = ?1 order by u.gmtModified desc")
    List<UserDevice> findByUserId(String userId);

    /**
     * 查询所有已采集到的设备类型（去除重复）
     * @return
     */
    @Query(value = "select distinct(u.deviceModel) from  UserDevice  u where u.deviceModel LIKE CONCAT('%',?1,'%') order by u.gmtModified desc")
    List<String> queryDeviceModel(String deviceModel);
}
