package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.user.dto.MerchantPermitDeviceDTO;
import cn.rankin.data.api.user.entity.MerchantPermitDevice;
import cn.rankin.userservice.repository.MerchantPermitDeviceRepository;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.LinkedHashMap;
import java.util.List;

import static cn.rankin.userservice.utils.DTOConverter.convert;

@Service
public class MerchantPermitDeviceService
{
    @Autowired
    private MerchantPermitDeviceRepository merchantPermitDeviceRepository;

    public APIResult<MerchantPermitDevice> findByMerchantId(String merchantId){
        List<MerchantPermitDevice> merchantPermitDevices = merchantPermitDeviceRepository.findByMerchantId(merchantId, BaseStatusEnum.NORMAL);
        return APIResult.ok(merchantPermitDevices);
    }

    @Transactional
    public APIResult<MerchantPermitDevice> create(MerchantPermitDeviceDTO dto) {
        MerchantPermitDevice entity = convert(dto);
        MerchantPermitDevice result = merchantPermitDeviceRepository.save(entity);

        if(null == result ){
            return APIResult.error(APICode.OPERATE_ERROR);
        }

        return APIResult.ok(result);
    }

    public APIResult<Page<MerchantPermitDevice>> search(MerchantPermitDevice simple, Integer pageNo, Integer pageSize, LinkedHashMap<String,BaseOrderEnum> sort) {
        Long count = merchantPermitDeviceRepository.count(simple);
        Page<MerchantPermitDevice> page = new Page<>(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<MerchantPermitDevice> list = merchantPermitDeviceRepository.find(simple, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(list);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<MerchantPermitDevice> update(MerchantPermitDeviceDTO dto) {
        MerchantPermitDevice entity = convert(dto);
        MerchantPermitDevice result = merchantPermitDeviceRepository.update(entity);

        if(null == result ){
            return APIResult.error(APICode.OPERATE_ERROR);
        }

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = merchantPermitDeviceRepository.updateStatusById(id, BaseStatusEnum.DEL);

        if (count == 0) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(true);
    }

    public APIResult<MerchantPermitDevice> getMerchantPermitDevice(String id) {
        if(StringUtils.isBlank(id)){
            return APIResult.error(APICode.OPERATE_ERROR);
        }
        MerchantPermitDevice result = merchantPermitDeviceRepository.find(id);
        return APIResult.ok(result);
    }
}
