package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.user.dto.UserDeviceDTO;
import cn.rankin.data.api.user.entity.UserDevice;
import cn.rankin.data.api.user.vo.DeviceModelVo;
import cn.rankin.userservice.repository.UserDeviceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class UserDeviceService
{
    @Autowired
    private UserDeviceRepository userDeviceRepository;

    /**
     * Find
     *
     * @return
     */
    public int opUserDevice(UserDeviceDTO uerDeviceDTO)
    {
        if (uerDeviceDTO == null)
        {
            return 0;
        }

        String uid = uerDeviceDTO.getUserId();
        String deviceCode = uerDeviceDTO.getDeviceCode();
        String deviceModel = uerDeviceDTO.getDeviceModel();
        String deviceName = uerDeviceDTO.getDeviceName();
        String deviceType = uerDeviceDTO.getDeviceType();
        String deviceMfrs = uerDeviceDTO.getDeviceMfrs();

        //查询用户设备信息
        UserDevice userDevice = userDeviceRepository.findUserDevice(uid, deviceName, deviceModel);
        if (userDevice != null)
        {
            //更新访问量
            int num = userDevice.getNum() + 1;
            int updateRet = userDeviceRepository.updateDevice(uid, deviceName, deviceModel, num);

            return updateRet;
        }
        else
        {
            //添加用户设备信息
            userDevice = new UserDevice();
            userDevice.setUserId(uid);
            userDevice.setGmtCreated(new Date());
            userDevice.setDeviceCode(deviceCode);
            userDevice.setDeviceModel(deviceModel);
            userDevice.setDeviceMfrs(deviceMfrs);
            userDevice.setDeviceName(deviceName);
            userDevice.setDeviceType(deviceType);
            userDevice.setNum(1);

            UserDevice userDeviceRet = userDeviceRepository.save(userDevice);
            if (userDeviceRet != null)
            {
                return 1;
            }
            return 0;
        }
    }

    /**
     *Find by user id
     * @param uid the user id
     * @return the list of user device
     */
    public List<UserDevice> findByUserId(String uid)
    {
        List<UserDevice> list = userDeviceRepository.findByUserId(uid);
        return list;
    }

    public APIResult<Page<UserDevice>> search(UserDevice simple, Integer pageNo, Integer pageSize, LinkedHashMap<String,BaseOrderEnum> sort) {
        Long count = userDeviceRepository.count(simple);
        Page<UserDevice> page = new Page<>(count, pageNo, pageSize);
        if (count == 0) {
            return APIResult.ok(page);
        }


        List<UserDevice> result = userDeviceRepository.find(simple, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(result);

        return APIResult.ok(page);

    }

    public APIResult<List<DeviceModelVo>> searchDeviceModel(String deviceModel) {
        List<String> models = userDeviceRepository.queryDeviceModel(deviceModel);
        List<DeviceModelVo> deviceModels = new ArrayList<>();
        models.forEach(model -> {
            DeviceModelVo vo = new DeviceModelVo();
            vo.setDeviceModel(model);
            deviceModels.add(vo);
        });

        return APIResult.ok(deviceModels);
    }
}
